export class Main extends Indicator {

    constructor() {
        super();

        this.setProperty(Property.SeparateWindow, true);

        this.addInput("FastEMA", InputType.int, 12);
        this.addInput("SlowEMA", InputType.int, 26);
        this.addInput("SignalSMA", InputType.int, 9);
        this.addInput("PriceType", InputType.PriceType, PriceType.Bid);

        this.buffers = {
            MacdBuffer : this.addBuffer(),
            SignalBuffer : this.addBuffer(),
        };
    }

    onInit() {
        const  {FastEMA, SlowEMA, SignalSMA} = this.getInputs();

        this.buffers.MacdBuffer
            .setShape(Shape.Histogram)
            .setColor(Color.Silver)
            .setWidth(2)
            .setLabel("MACD")
            .setShortName("MACD(" + FastEMA + "," + SlowEMA + "," + SignalSMA + ")")
            .setDigits(Current.Digits+1)
        ;

        this.buffers.SignalBuffer
            .setShape(Shape.Line)
            .setColor(Color.Red)
            .setLabel("Signal")
            .setDrawBegin(SignalSMA)
            .setShortName("MACD(" + FastEMA + "," + SlowEMA + "," + SignalSMA + ")")
            .setDigits(Current.Digits+1)
        ;
    }

    async onUpdate() {
        const  {FastEMA, SlowEMA, SignalSMA, PriceType} = this.getInputs(),
            {MacdBuffer, SignalBuffer} = this.buffers,
            {Close} = await Bar.load([Bar.Mode.Close], PriceType),
            barSize = Close.length;

        let limit, i, list1, list2, countedBars = Bar.counted();

        if (countedBars > 0) countedBars--;

        limit = barSize - countedBars;

        list1 = await Indicators.MA(null, 0, FastEMA, 0, Averaging.Exponential, AppliedPrice.Close, PriceType);
        list2 = await Indicators.MA(null, 0, SlowEMA, 0, Averaging.Exponential, AppliedPrice.Close, PriceType);

        for (i = 0; i < limit; i++) {
            MacdBuffer.set(i, (list1[i] - list2[i]));
        }
        list1 = Indicators.MAOnArray(MacdBuffer, barSize, SignalSMA, 0, Averaging.Simple, i);
        SignalBuffer.fill(list1);
    }
}